"""
Plots of how work time was spent.

Requires matplotlib.
"""

# Copyright 2008, Markus Demleitner.  This file is under the GPL,
# see COPYING for details.

from datetime import date
import contextlib
import sys

import numpy
from matplotlib import pyplot

from pftf import due
from pftf import settings
from pftf import times


@contextlib.contextmanager
def fig_writeto(dest_name):
	"""starts a new pyplot figure and saves it to dest_name once
	the controlled section is left.
	"""
	pyplot.figure(figsize=(9,3))
	pyplot.axes()
	yield
	pyplot.legend()
	pyplot.savefig(dest_name)
	pyplot.close()


def getColorForLabel(label):
	col = settings.get("classCols", label)
	return fill_style.Plain(bgcolor=getattr(color, col, color.white))


def makeDistPieGraph(times, ax):
	ax.set_position((0.57, 0.45, 0.55, 0.55))

	times = times.getWorkTimeDist().items()
	total = sum(t[1] for t in times)
	ax.pie(
		[t[1] for t in times],
		colors=[settings.get("classCols", t[0]) for t in times],
		autopct="%2.0f%%",
		pctdistance=0.9)


def makeMonthBarGraph(times, ax):
	globalDist = times.getWorkTimeDist()
	globalDist.pop("Transfer", None)
	classesPresent = [label 
		for label in settings.get("wClasses") 
		if label in globalDist]
	classesSkipped = set(globalDist)-set(settings.get("wClasses"))
	if classesSkipped:
		print(f"Not plotted: {classesSkipped}")

	days = list(times.iterdays())
	seqs = dict((c, []) for c in classesPresent)

	for day in days:
		dist = times.getForDay(day).getWorkTimes()
		for label in classesPresent:
			seqs[label].append(dist.get(label, 0))

	curBottom = numpy.zeros(len(days))
	dayNumbers = [d.day for d in days]
	ax.set_xticks(dayNumbers)
	
	for label in classesPresent:
		ax.bar(dayNumbers, 
				seqs[label], 
				color=settings.get("classCols", label),
				bottom=curBottom,
				label=label)
		curBottom += seqs[label]



def makeMonthGraph(opts):
	"""generates a graphics for the work given in the Times object times,
	where the plot is laid out such that it looks ok for a month.
	"""
	with fig_writeto(opts.outputName):
		tForMonth = times.Times.forMonth(opts)
		fig = pyplot.figure(figsize=(10, 5))

		makeDistPieGraph(tForMonth, fig.add_subplot(2, 2, 2, zorder=1))
		makeMonthBarGraph(tForMonth, fig.add_subplot(1, 1, 1, zorder=0))
#		lineY = settings.get("plot", "height")-10
#		canvas.show(10, lineY, "/b/17Work in %d-%02d: %.1f hours"%(
#			opts.year, opts.month, tForMonth.getWorkTime()))
#		canvas.show(10, lineY-20, 
#			"/17Due was %.1f"%due.getHoursForMonth(opts.month, opts.year, 
#			times=tForMonth))
