"""
Generator for a custom cli binary.
"""

import os
import shutil
import tempfile

from pftf import settings


cTemplate = """/* Generated code, do not edit */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

char *labels[] = {
%(labels)s,
	"quit",
	NULL,
};

char *destFName = "%(destFName)s";

char *progname;

int isValidLabel(char *label)
{ /* a linear search will do */
	char **cur = labels;

	while (*cur) {
		if (!strcmp(*cur, label)) {
			return 1;
		}
		cur++;
	}
	return 0;
}

void usage()
{
	char **cur=labels;

	fprintf(stderr, "Usage: %%s <label>\\n", progname);
	fprintf(stderr, "Where label is one of:\\n");
	while (*cur) {
		fprintf(stderr, "%%s ", *cur);
		cur++;
	}
	fprintf(stderr, "\\n");
	exit(0);
}

#define TIME_LEN 30

void stamp(char *label) {
	struct tm *now;
	time_t t;
	char timeStr[TIME_LEN];
	FILE *f;

	t = time(NULL);
	now = localtime(&t);
	strftime(timeStr, TIME_LEN, "%(timestampFormat)s", now);
	f = fopen(destFName, "a");
	if (strcmp(label, "quit")) {
		fprintf(f, "%%s\\t%%s\\n", timeStr, label);
	} else {
		fprintf(f, "%%s\\n", timeStr, label);
	}
	close(f);
}

int main(int argc, char **argv)
{
	progname = argv[0];
	if (argc!=2 || (argc==2 && !isValidLabel(argv[1]))) {
		usage();
	}
	stamp(argv[1]);
	return 0;
}
"""


def getSource():
	return cTemplate%{
		"labels": ", ".join(['"%s"'%c for c in settings.get("wClasses")]),
		"destFName": os.path.join(settings.dataDir, 
			settings.get("filenames", "piningFile")),
		"timestampFormat": settings.get("isoformat"),
	}

def makeBinary(source, destName):
	owd = os.getcwd()
	dir = tempfile.mkdtemp("build", "tmp", owd)
	os.chdir(dir)
	f = open("%s.c"%destName, "w")
	f.write(source)
	f.close()
	os.system("make %s"%destName)
	os.system("strip %s"%destName)
	os.rename(destName, os.path.join(owd, destName))
	os.chdir(owd)
	shutil.rmtree(dir)


def main(opts=None):
	src = getSource()
	makeBinary(src, "pinestamp")


if __name__=="__main__":
	main()
